ALTER EXTENSION pgtap ADD FUNCTION pg_version();
ALTER EXTENSION pgtap ADD FUNCTION pg_version_num();
ALTER EXTENSION pgtap ADD FUNCTION os_name();
ALTER EXTENSION pgtap ADD FUNCTION pgtap_version();
ALTER EXTENSION pgtap ADD FUNCTION plan( integer );
ALTER EXTENSION pgtap ADD FUNCTION no_plan();
ALTER EXTENSION pgtap ADD FUNCTION _get ( text );
ALTER EXTENSION pgtap ADD FUNCTION _get_latest ( text );
ALTER EXTENSION pgtap ADD FUNCTION _get_latest ( text, integer );
ALTER EXTENSION pgtap ADD FUNCTION _get_note ( text );
ALTER EXTENSION pgtap ADD FUNCTION _get_note ( integer );
ALTER EXTENSION pgtap ADD FUNCTION _set ( text, integer, text );
ALTER EXTENSION pgtap ADD FUNCTION _set ( text, integer );
ALTER EXTENSION pgtap ADD FUNCTION _set ( integer, integer );
ALTER EXTENSION pgtap ADD FUNCTION _add ( text, integer, text );
ALTER EXTENSION pgtap ADD FUNCTION _add ( text, integer );
ALTER EXTENSION pgtap ADD FUNCTION add_result ( bool, bool, text, text, text );
ALTER EXTENSION pgtap ADD FUNCTION num_failed ();
ALTER EXTENSION pgtap ADD FUNCTION _finish ( INTEGER, INTEGER, INTEGER);
ALTER EXTENSION pgtap ADD FUNCTION finish ();
ALTER EXTENSION pgtap ADD FUNCTION diag ( msg text );
ALTER EXTENSION pgtap ADD FUNCTION diag ( msg anyelement );
ALTER EXTENSION pgtap ADD FUNCTION diag( VARIADIC text[] );
ALTER EXTENSION pgtap ADD FUNCTION diag( VARIADIC anyarray );
ALTER EXTENSION pgtap ADD FUNCTION ok ( boolean, text );
ALTER EXTENSION pgtap ADD FUNCTION ok ( boolean );
ALTER EXTENSION pgtap ADD FUNCTION is (anyelement, anyelement, text);
ALTER EXTENSION pgtap ADD FUNCTION is (anyelement, anyelement);
ALTER EXTENSION pgtap ADD FUNCTION isnt (anyelement, anyelement, text);
ALTER EXTENSION pgtap ADD FUNCTION isnt (anyelement, anyelement);
ALTER EXTENSION pgtap ADD FUNCTION _alike ( BOOLEAN, ANYELEMENT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION matches ( anyelement, text, text );
ALTER EXTENSION pgtap ADD FUNCTION matches ( anyelement, text );
ALTER EXTENSION pgtap ADD FUNCTION imatches ( anyelement, text, text );
ALTER EXTENSION pgtap ADD FUNCTION imatches ( anyelement, text );
ALTER EXTENSION pgtap ADD FUNCTION alike ( anyelement, text, text );
ALTER EXTENSION pgtap ADD FUNCTION alike ( anyelement, text );
ALTER EXTENSION pgtap ADD FUNCTION ialike ( anyelement, text, text );
ALTER EXTENSION pgtap ADD FUNCTION ialike ( anyelement, text );
ALTER EXTENSION pgtap ADD FUNCTION _unalike ( BOOLEAN, ANYELEMENT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION doesnt_match ( anyelement, text, text );
ALTER EXTENSION pgtap ADD FUNCTION doesnt_match ( anyelement, text );
ALTER EXTENSION pgtap ADD FUNCTION doesnt_imatch ( anyelement, text, text );
ALTER EXTENSION pgtap ADD FUNCTION doesnt_imatch ( anyelement, text );
ALTER EXTENSION pgtap ADD FUNCTION unalike ( anyelement, text, text );
ALTER EXTENSION pgtap ADD FUNCTION unalike ( anyelement, text );
ALTER EXTENSION pgtap ADD FUNCTION unialike ( anyelement, text, text );
ALTER EXTENSION pgtap ADD FUNCTION unialike ( anyelement, text );
ALTER EXTENSION pgtap ADD FUNCTION cmp_ok (anyelement, text, anyelement, text);
ALTER EXTENSION pgtap ADD FUNCTION cmp_ok (anyelement, text, anyelement);
ALTER EXTENSION pgtap ADD FUNCTION pass ( text );
ALTER EXTENSION pgtap ADD FUNCTION pass ();
ALTER EXTENSION pgtap ADD FUNCTION fail ( text );
ALTER EXTENSION pgtap ADD FUNCTION fail ();
ALTER EXTENSION pgtap ADD FUNCTION todo ( why text, how_many int );
ALTER EXTENSION pgtap ADD FUNCTION todo ( how_many int, why text );
ALTER EXTENSION pgtap ADD FUNCTION todo ( why text );
ALTER EXTENSION pgtap ADD FUNCTION todo ( how_many int );
ALTER EXTENSION pgtap ADD FUNCTION todo_start (text);
ALTER EXTENSION pgtap ADD FUNCTION todo_start ();
ALTER EXTENSION pgtap ADD FUNCTION in_todo ();
ALTER EXTENSION pgtap ADD FUNCTION todo_end ();
ALTER EXTENSION pgtap ADD FUNCTION _todo();
ALTER EXTENSION pgtap ADD FUNCTION skip ( why text, how_many int );
ALTER EXTENSION pgtap ADD FUNCTION skip ( text );
ALTER EXTENSION pgtap ADD FUNCTION skip( int, text );
ALTER EXTENSION pgtap ADD FUNCTION skip( int );
ALTER EXTENSION pgtap ADD FUNCTION _query( TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_ok ( TEXT, CHAR(5), TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_ok ( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_ok ( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_ok ( TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_ok ( TEXT, int4, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_ok ( TEXT, int4, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_ok ( TEXT, int4 );
ALTER EXTENSION pgtap ADD FUNCTION lives_ok ( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION lives_ok ( TEXT );
ALTER EXTENSION pgtap ADD FUNCTION performs_ok ( TEXT, NUMERIC, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION performs_ok ( TEXT, NUMERIC );
ALTER EXTENSION pgtap ADD FUNCTION performs_within ( TEXT, NUMERIC, NUMERIC, INT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION performs_within ( TEXT, NUMERIC, NUMERIC, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION performs_within ( TEXT, NUMERIC, NUMERIC, INT );
ALTER EXTENSION pgtap ADD FUNCTION performs_within ( TEXT, NUMERIC, NUMERIC );
ALTER EXTENSION pgtap ADD FUNCTION _rexists ( CHAR, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _rexists ( CHAR, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_table ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_table ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_table ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_table ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_table ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_table ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_view ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_view ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_view ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_view ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_view ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_view ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_sequence ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_sequence ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_sequence ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_sequence ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_sequence ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_sequence ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _cexists ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _cexists ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_column ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_column ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_column ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_column ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_column ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_column ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _col_is_null ( NAME, NAME, NAME, TEXT, bool );
ALTER EXTENSION pgtap ADD FUNCTION _col_is_null ( NAME, NAME, TEXT, bool );
ALTER EXTENSION pgtap ADD FUNCTION col_not_null ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_not_null ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_not_null ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION col_is_null ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_null ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION col_is_null ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION display_type ( OID, INTEGER );
ALTER EXTENSION pgtap ADD FUNCTION display_type ( NAME, OID, INTEGER );
ALTER EXTENSION pgtap ADD FUNCTION _get_col_type ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _get_col_type ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _get_col_ns_type ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _quote_ident_like(TEXT, TEXT);
ALTER EXTENSION pgtap ADD FUNCTION col_type_is ( NAME, NAME, NAME, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_type_is ( NAME, NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_type_is ( NAME, NAME, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_type_is ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_type_is ( NAME, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_type_is ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _has_def ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _has_def ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION col_has_default ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_has_default ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_has_default ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION col_hasnt_default ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_hasnt_default ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_hasnt_default ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _def_is( TEXT, TEXT, anyelement, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _cdi ( NAME, NAME, NAME, anyelement, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _cdi ( NAME, NAME, anyelement, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _cdi ( NAME, NAME, anyelement );
ALTER EXTENSION pgtap ADD FUNCTION col_default_is ( NAME, NAME, NAME, anyelement, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_default_is ( NAME, NAME, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_default_is ( NAME, NAME, anyelement, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_default_is ( NAME, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_default_is ( NAME, NAME, anyelement );
ALTER EXTENSION pgtap ADD FUNCTION col_default_is ( NAME, NAME, text );
ALTER EXTENSION pgtap ADD FUNCTION _hasc ( NAME, NAME, CHAR );
ALTER EXTENSION pgtap ADD FUNCTION _hasc ( NAME, CHAR );
ALTER EXTENSION pgtap ADD FUNCTION has_pk ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_pk ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_pk ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_pk ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_pk ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_pk ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _ident_array_to_string( name[], text );
ALTER EXTENSION pgtap ADD FUNCTION _pg_sv_column_array( OID, SMALLINT[] );
ALTER EXTENSION pgtap ADD FUNCTION _pg_sv_table_accessible( OID, OID );
ALTER EXTENSION pgtap ADD VIEW pg_all_foreign_keys;
ALTER EXTENSION pgtap ADD FUNCTION _keys ( NAME, NAME, CHAR );
ALTER EXTENSION pgtap ADD FUNCTION _keys ( NAME, CHAR );
ALTER EXTENSION pgtap ADD FUNCTION _ckeys ( NAME, NAME, CHAR );
ALTER EXTENSION pgtap ADD FUNCTION _ckeys ( NAME, CHAR );
ALTER EXTENSION pgtap ADD FUNCTION col_is_pk ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_pk ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_pk ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION col_is_pk ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_pk ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_pk ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_pk ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_pk ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_pk ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_pk ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_pk ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_pk ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_fk ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_fk ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_fk ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_fk ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_fk ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_fk ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _fkexists ( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _fkexists ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION col_is_fk ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_fk ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_fk ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION col_is_fk ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_fk ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_fk ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_fk ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_fk ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_fk ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_fk ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_fk ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_isnt_fk ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_unique ( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_unique ( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_unique ( TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _constraint ( NAME, NAME, CHAR, NAME[], TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _constraint ( NAME, CHAR, NAME[], TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_unique ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_unique ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_unique ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION col_is_unique ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_unique ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_is_unique ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_check ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_check ( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_check ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION col_has_check ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_has_check ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_has_check ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION col_has_check ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_has_check ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION col_has_check ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION fk_ok ( NAME, NAME, NAME[], NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION fk_ok ( NAME, NAME[], NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION fk_ok ( NAME, NAME, NAME[], NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION fk_ok ( NAME, NAME[], NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION fk_ok ( NAME, NAME, NAME, NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION fk_ok ( NAME, NAME, NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION fk_ok ( NAME, NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION fk_ok ( NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD VIEW tap_funky;
ALTER EXTENSION pgtap ADD FUNCTION _got_func ( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _got_func ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _got_func ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _got_func ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_function ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_function( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION has_function ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_function( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_function ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_function( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION has_function( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_function( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_function ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_function( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_function ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_function( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_function ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_function( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_function( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_function( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _pg_sv_type_array( OID[] );
ALTER EXTENSION pgtap ADD FUNCTION can ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION can ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION can ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION can ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _ikeys( NAME, NAME, NAME);
ALTER EXTENSION pgtap ADD FUNCTION _ikeys( NAME, NAME);
ALTER EXTENSION pgtap ADD FUNCTION _have_index( NAME, NAME, NAME);
ALTER EXTENSION pgtap ADD FUNCTION _have_index( NAME, NAME);
ALTER EXTENSION pgtap ADD FUNCTION _iexpr( NAME, NAME, NAME);
ALTER EXTENSION pgtap ADD FUNCTION _iexpr( NAME, NAME);
ALTER EXTENSION pgtap ADD FUNCTION has_index ( NAME, NAME, NAME, NAME[], text );
ALTER EXTENSION pgtap ADD FUNCTION has_index ( NAME, NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION has_index ( NAME, NAME, NAME, NAME, text );
ALTER EXTENSION pgtap ADD FUNCTION has_index ( NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_index ( NAME, NAME, NAME[], text );
ALTER EXTENSION pgtap ADD FUNCTION has_index ( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _is_schema( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_index ( NAME, NAME, NAME, text );
ALTER EXTENSION pgtap ADD FUNCTION has_index ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_index ( NAME, NAME, text );
ALTER EXTENSION pgtap ADD FUNCTION has_index ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_index ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_index ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_index ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_index ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION index_is_unique ( NAME, NAME, NAME, text );
ALTER EXTENSION pgtap ADD FUNCTION index_is_unique ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION index_is_unique ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION index_is_unique ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION index_is_primary ( NAME, NAME, NAME, text );
ALTER EXTENSION pgtap ADD FUNCTION index_is_primary ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION index_is_primary ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION index_is_primary ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_clustered ( NAME, NAME, NAME, text );
ALTER EXTENSION pgtap ADD FUNCTION is_clustered ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_clustered ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_clustered ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION index_is_type ( NAME, NAME, NAME, NAME, text );
ALTER EXTENSION pgtap ADD FUNCTION index_is_type ( NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION index_is_type ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION index_is_type ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _trig ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _trig ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_trigger ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_trigger ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_trigger ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_trigger ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_trigger ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_trigger ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_trigger ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_trigger ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION trigger_is ( NAME, NAME, NAME, NAME, NAME, text );
ALTER EXTENSION pgtap ADD FUNCTION trigger_is ( NAME, NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION trigger_is ( NAME, NAME, NAME, text );
ALTER EXTENSION pgtap ADD FUNCTION trigger_is ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_schema( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_schema( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_schema( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_schema( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_tablespace( NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_tablespace( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_tablespace( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_tablespace( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_tablespace( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _has_type( NAME, NAME, CHAR[] );
ALTER EXTENSION pgtap ADD FUNCTION _has_type( NAME, CHAR[] );
ALTER EXTENSION pgtap ADD FUNCTION has_type( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_type( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_type( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_type( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_type( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_type( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_type( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_type( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_domain( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_domain( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_domain( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_domain( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_domain( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_domain( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_domain( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_domain( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_enum( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_enum( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_enum( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_enum( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_enum( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_enum( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_enum( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_enum( NAME );
ALTER EXTENSION pgtap ADD FUNCTION enum_has_labels( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION enum_has_labels( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION enum_has_labels( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION enum_has_labels( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _has_role( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_role( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_role( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_role( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_role( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _has_user( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_user( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_user( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_user( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_user( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _is_super( NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_superuser( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_superuser( NAME );
ALTER EXTENSION pgtap ADD FUNCTION isnt_superuser( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION isnt_superuser( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _has_group( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_group( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_group( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_group( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_group( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _grolist ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_member_of( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_member_of( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_member_of( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION is_member_of( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _cmp_types(oid, name);
ALTER EXTENSION pgtap ADD FUNCTION _cast_exists ( NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _cast_exists ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _cast_exists ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_cast ( NAME, NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_cast ( NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_cast ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_cast ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_cast ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_cast ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_cast ( NAME, NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_cast ( NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_cast ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_cast ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_cast ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_cast ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _expand_context( char );
ALTER EXTENSION pgtap ADD FUNCTION _get_context( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION cast_context_is( NAME, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION cast_context_is( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _op_exists ( NAME, NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _op_exists ( NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _op_exists ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_operator ( NAME, NAME, NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_operator ( NAME, NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_operator ( NAME, NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_operator ( NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_operator ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_operator ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_leftop ( NAME, NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_leftop ( NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_leftop ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_leftop ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_leftop ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_leftop ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_rightop ( NAME, NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_rightop ( NAME, NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_rightop ( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_rightop ( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_rightop ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_rightop ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _are ( text, name[], name[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION tablespaces_are ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION tablespaces_are ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION schemas_are ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION schemas_are ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _extras ( CHAR, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _extras ( CHAR, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _missing ( CHAR, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _missing ( CHAR, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION tables_are ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION tables_are ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION tables_are ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION tables_are ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION views_are ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION views_are ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION views_are ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION views_are ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION sequences_are ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION sequences_are ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION sequences_are ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION sequences_are ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION functions_are ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION functions_are ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION functions_are ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION functions_are ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION indexes_are( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION indexes_are( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION indexes_are( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION indexes_are( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION users_are( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION users_are( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION groups_are( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION groups_are( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION languages_are( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION languages_are( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _is_trusted( NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_language( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_language( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_language( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_language( NAME );
ALTER EXTENSION pgtap ADD FUNCTION language_is_trusted( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION language_is_trusted( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _opc_exists( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_opclass( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_opclass( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_opclass( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_opclass( NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_opclass( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_opclass( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_opclass( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_opclass( NAME );
ALTER EXTENSION pgtap ADD FUNCTION opclasses_are ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION opclasses_are ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION opclasses_are ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION opclasses_are ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION rules_are( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION rules_are( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION rules_are( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION rules_are( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _is_instead( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _is_instead( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_rule( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_rule( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION has_rule( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION has_rule( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_rule( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_rule( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_rule( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION hasnt_rule( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION rule_is_instead( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION rule_is_instead( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION rule_is_instead( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION rule_is_instead( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _expand_on( char );
ALTER EXTENSION pgtap ADD FUNCTION _contract_on( TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _rule_on( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _rule_on( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION rule_is_on( NAME, NAME, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION rule_is_on( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION rule_is_on( NAME, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION rule_is_on( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _nosuch( NAME, NAME, NAME[]);
ALTER EXTENSION pgtap ADD FUNCTION _func_compare( NAME, NAME, NAME[], anyelement, anyelement, TEXT);
ALTER EXTENSION pgtap ADD FUNCTION _func_compare( NAME, NAME, NAME[], boolean, TEXT);
ALTER EXTENSION pgtap ADD FUNCTION _func_compare( NAME, NAME, anyelement, anyelement, TEXT);
ALTER EXTENSION pgtap ADD FUNCTION _func_compare( NAME, NAME, boolean, TEXT);
ALTER EXTENSION pgtap ADD FUNCTION _lang ( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _lang ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _lang ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _lang ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION function_lang_is( NAME, NAME, NAME[], NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_lang_is( NAME, NAME, NAME[], NAME );
ALTER EXTENSION pgtap ADD FUNCTION function_lang_is( NAME, NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_lang_is( NAME, NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION function_lang_is( NAME, NAME[], NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_lang_is( NAME, NAME[], NAME );
ALTER EXTENSION pgtap ADD FUNCTION function_lang_is( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_lang_is( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _returns ( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _returns ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _returns ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _returns ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION function_returns( NAME, NAME, NAME[], TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_returns( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_returns( NAME, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_returns( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_returns( NAME, NAME[], TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_returns( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_returns( NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION function_returns( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _definer ( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _definer ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _definer ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _definer ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_definer ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_definer( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION is_definer ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_definer( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_definer ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_definer( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION is_definer( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_definer( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _agg ( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _agg ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _agg ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _agg ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_aggregate ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_aggregate( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION is_aggregate ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_aggregate( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_aggregate ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_aggregate( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION is_aggregate( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_aggregate( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _strict ( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _strict ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _strict ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _strict ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_strict ( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_strict( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION is_strict ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_strict( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION is_strict ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_strict( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION is_strict( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_strict( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _expand_vol( char );
ALTER EXTENSION pgtap ADD FUNCTION _refine_vol( text );
ALTER EXTENSION pgtap ADD FUNCTION _vol ( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _vol ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _vol ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _vol ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION volatility_is( NAME, NAME, NAME[], TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION volatility_is( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION volatility_is( NAME, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION volatility_is( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION volatility_is( NAME, NAME[], TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION volatility_is( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION volatility_is( NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION volatility_is( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION check_test( TEXT, BOOLEAN, TEXT, TEXT, TEXT, BOOLEAN );
ALTER EXTENSION pgtap ADD FUNCTION check_test( TEXT, BOOLEAN, TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION check_test( TEXT, BOOLEAN, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION check_test( TEXT, BOOLEAN, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION check_test( TEXT, BOOLEAN );
ALTER EXTENSION pgtap ADD FUNCTION findfuncs( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION findfuncs( TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _runem( text[], boolean );
ALTER EXTENSION pgtap ADD FUNCTION _is_verbose();
ALTER EXTENSION pgtap ADD FUNCTION do_tap( name, text );
ALTER EXTENSION pgtap ADD FUNCTION do_tap( name );
ALTER EXTENSION pgtap ADD FUNCTION do_tap( text );
ALTER EXTENSION pgtap ADD FUNCTION do_tap( );
ALTER EXTENSION pgtap ADD FUNCTION _currtest();
ALTER EXTENSION pgtap ADD FUNCTION _cleanup();
ALTER EXTENSION pgtap ADD FUNCTION diag_test_name(TEXT);
ALTER EXTENSION pgtap ADD FUNCTION _runner( text[], text[], text[], text[], text[] );
ALTER EXTENSION pgtap ADD FUNCTION runtests( NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION runtests( NAME );
ALTER EXTENSION pgtap ADD FUNCTION runtests( TEXT );
ALTER EXTENSION pgtap ADD FUNCTION runtests( );
ALTER EXTENSION pgtap ADD FUNCTION _temptable ( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _temptable ( anyarray, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _temptypes( TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _docomp( TEXT, TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _relcomp( TEXT, TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _relcomp( TEXT, anyarray, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_eq( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_eq( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_eq( TEXT, anyarray, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_eq( TEXT, anyarray );
ALTER EXTENSION pgtap ADD FUNCTION bag_eq( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION bag_eq( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION bag_eq( TEXT, anyarray, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION bag_eq( TEXT, anyarray );
ALTER EXTENSION pgtap ADD FUNCTION _do_ne( TEXT, TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _relne( TEXT, TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION _relne( TEXT, anyarray, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_ne( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_ne( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_ne( TEXT, anyarray, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_ne( TEXT, anyarray );
ALTER EXTENSION pgtap ADD FUNCTION bag_ne( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION bag_ne( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION bag_ne( TEXT, anyarray, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION bag_ne( TEXT, anyarray );
ALTER EXTENSION pgtap ADD FUNCTION _relcomp( TEXT, TEXT, TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_has( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_has( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION bag_has( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION bag_has( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_hasnt( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION set_hasnt( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION bag_hasnt( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION bag_hasnt( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( refcursor, refcursor, text );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( refcursor, refcursor );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( TEXT, anyarray, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( TEXT, anyarray );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( TEXT, refcursor, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( TEXT, refcursor );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( refcursor, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( refcursor, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( refcursor, anyarray, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_eq( refcursor, anyarray );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( refcursor, refcursor, text );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( refcursor, refcursor );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( TEXT, anyarray, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( TEXT, anyarray );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( TEXT, refcursor, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( TEXT, refcursor );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( refcursor, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( refcursor, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( refcursor, anyarray, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION results_ne( refcursor, anyarray );
ALTER EXTENSION pgtap ADD FUNCTION isa_ok( anyelement, regtype, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION isa_ok( anyelement, regtype );
ALTER EXTENSION pgtap ADD FUNCTION is_empty( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION is_empty( TEXT );
ALTER EXTENSION pgtap ADD FUNCTION collect_tap( VARIADIC text[] );
ALTER EXTENSION pgtap ADD FUNCTION collect_tap( VARCHAR[] );
ALTER EXTENSION pgtap ADD FUNCTION _tlike ( BOOLEAN, TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_like ( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_like ( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_ilike ( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_ilike ( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_matching ( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_matching ( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_imatching ( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION throws_imatching ( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION roles_are( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION roles_are( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _types_are ( NAME, NAME[], TEXT, CHAR[] );
ALTER EXTENSION pgtap ADD FUNCTION types_are ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION types_are ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _types_are ( NAME[], TEXT, CHAR[] );
ALTER EXTENSION pgtap ADD FUNCTION types_are ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION types_are ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION domains_are ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domains_are ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION domains_are ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domains_are ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION enums_are ( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION enums_are ( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION enums_are ( NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION enums_are ( NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _dexists ( NAME, NAME );
ALTER EXTENSION pgtap ADD FUNCTION _dexists ( NAME );
ALTER EXTENSION pgtap ADD FUNCTION _get_dtype( NAME, TEXT, BOOLEAN );
ALTER EXTENSION pgtap ADD FUNCTION _get_dtype( NAME );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_is( NAME, TEXT, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_is( NAME, TEXT, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_is( NAME, TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_is( NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_is( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_is( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_isnt( NAME, TEXT, NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_isnt( NAME, TEXT, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_isnt( NAME, TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_isnt( NAME, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_isnt( TEXT, TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION domain_type_isnt( TEXT, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION row_eq( TEXT, anyelement, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION row_eq( TEXT, anyelement );
ALTER EXTENSION pgtap ADD FUNCTION triggers_are( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION triggers_are( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION triggers_are( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION triggers_are( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _areni ( text, text[], text[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION casts_are ( TEXT[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION casts_are ( TEXT[] );
ALTER EXTENSION pgtap ADD FUNCTION display_oper ( NAME, OID );
ALTER EXTENSION pgtap ADD FUNCTION operators_are( NAME, TEXT[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION operators_are ( NAME, TEXT[] );
ALTER EXTENSION pgtap ADD FUNCTION operators_are( TEXT[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION operators_are ( TEXT[] );
ALTER EXTENSION pgtap ADD FUNCTION columns_are( NAME, NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION columns_are( NAME, NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION columns_are( NAME, NAME[], TEXT );
ALTER EXTENSION pgtap ADD FUNCTION columns_are( NAME, NAME[] );
ALTER EXTENSION pgtap ADD FUNCTION _get_db_owner( NAME );
ALTER EXTENSION pgtap ADD FUNCTION db_owner_is ( NAME, NAME, TEXT );
ALTER EXTENSION pgtap ADD FUNCTION db_owner_is ( NAME, NAME );
